@@ -5,7 +5,6 @@
 #pragma once
 
 #include "config.h"
-#include "image.h"
 
 /** Canvas context. */
 struct canvas;
@@ -33,6 +32,12 @@ enum canvas_scale {
 /** Corner position. */
 enum canvas_corner { cc_top_right, cc_bottom_left, cc_bottom_right };
 
+/** Meta data info table. */
+struct info_table {
+    const char* key;
+    const char* value;
+};
+
 /**
  * Initialize canvas.
  * @param ctx canvas context
@@ -50,9 +55,11 @@ void canvas_free(struct canvas* ctx);
  * Resize canvas window.
  * @param ctx canvas context
  * @param width,height new window size
+ * @param scale window scale factor
  * @return true if it was the first resize
  */
-bool canvas_resize_window(struct canvas* ctx, size_t width, size_t height);
+bool canvas_resize_window(struct canvas* ctx, size_t width, size_t height,
+                          size_t scale);
 
 /**
  * Reset image position, size and scale.
@@ -97,13 +104,14 @@ void canvas_print_line(const struct canvas* ctx, argb_t* wnd,
                        enum canvas_corner corner, const char* text);
 
 /**
- * Print meta info table on canvas.
+ * Print meta info table on the left top corner.
  * @param ctx canvas context
  * @param wnd window buffer
- * @param info meta info to print
+ * @param num total number of lines
+ * @param info meta data table to print
  */
-void canvas_print_meta(const struct canvas* ctx, argb_t* wnd,
-                       const struct meta* info);
+void canvas_print_info(const struct canvas* ctx, argb_t* wnd, size_t num,
+                       const struct info_table* info);
 
 /**
  * Move viewport.
