@@ -335,10 +335,11 @@ done:
     return rc;
 }
 
-bool sway_add_rules(int ipc, const char* app, int x, int y)
+bool sway_add_rules(int ipc, const char* app, int x, int y, bool absolute)
 {
     char move[64];
-    snprintf(move, sizeof(move), "move position %i %i", x, y);
+    snprintf(move, sizeof(move), "move %s position %i %i",
+             absolute ? "absolute" : "", x, y);
     return ipc_command(ipc, app, "floating enable") &&
         ipc_command(ipc, app, move);
 }
